package org.maltparser;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.jar.Attributes;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;

import org.maltparser.core.config.Configuration;
import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.options.OptionManager;
import org.maltparser.core.plugin.PluginLoader;

/**
 * MaltConsoleEngine controls the MaltParser system using the console version. 
 * 
 * @author Johan Hall
 * @since 1.0
**/
public class MaltConsoleEngine {
	private static String version;
	private static String buildDate;
	private static Attributes manifestAttributes;
	private static Logger systemLogger = Logger.getLogger("System");
	private Level systemVerbosityLevel;
	private static OptionManager optionManager = new OptionManager();
	private static PluginLoader pluginLoader = new PluginLoader();
	
	/**
	 * Creates a MaltConsoleEngine object
	 * 
	 * @param args command-line arguments
	 */
	public MaltConsoleEngine(String[] args) {
		/* System verbosity (Standard Out) */
		if (System.getProperty("Malt.verbosity") != null) {
			setSystemVerbosityLevel(System.getProperty("Malt.verbosity").toUpperCase());
		} else {
			setSystemVerbosityLevel("INFO");
		}
		
		ConsoleAppender consoleAppender = new ConsoleAppender(new PatternLayout("%m"));
		consoleAppender.setEncoding("UTF-16");
		consoleAppender.setThreshold(getSystemVerbosityLevel());
		systemLogger.addAppender(consoleAppender);
		systemLogger.setLevel(getSystemVerbosityLevel());
		try {
			getManifestInfo();
			/* Option and Plug-in management */
			//optionManager.parseOptionDescriptionXMLfile(getClass().getResource("/appdata/options.xml"));
			optionManager.loadOptionDescriptionFile();
			File maltJarPath = new File(new File(System.getProperty("java.class.path")).getAbsolutePath());
			if (maltJarPath != null) {
				pluginLoader.loadPlugins(new File(maltJarPath.getParent()+"/plugin"));
			}
			optionManager.generateMaps();
			optionManager.parseCommandLine(args);
			
			/* Update the verbosity level according to the verbosity option */
			String verbosity = (String)optionManager.getOptionValue("system", "verbosity");
			if (verbosity != null) {
				setSystemVerbosityLevel(verbosity.toUpperCase());
				consoleAppender.setThreshold(getSystemVerbosityLevel());
				systemLogger.setLevel(getSystemVerbosityLevel());
			}
		} catch (MaltChainedException e) {
			if (systemLogger.isDebugEnabled()) {
				systemLogger.debug("",e);
			} else {
				systemLogger.error(e.getMessageChain());
			}
			System.exit(1);
		}
	}
	
	/**
	 * Starts the console engine.
	 */
	public void startEngine() {
		try {
			
			/* Help or reading the option file */
			if (optionManager.getNumberOfOptionValues(0) == 0) {
				systemLogger.info(header());
				systemLogger.info(shortHelp());
				return;
			} else if (optionManager.getOptionValue("system", "help") != null) {
				systemLogger.info(header());
				systemLogger.info(optionManager.getOptionDescriptions());
				return;
			} else {
				if (optionManager.getOptionValue("system", "option_file") != null && optionManager.getOptionValue("system", "option_file").toString().length() > 0) {
					optionManager.parseOptionInstanceXMLfile((String)optionManager.getOptionValue("system", "option_file"));
				}
			}
			maltParser();
		} catch (MaltChainedException e) {
			if (systemLogger.isDebugEnabled()) {
				systemLogger.debug("",e);
			} else {
				systemLogger.error(e.getMessageChain());
			}
			System.exit(1);
		}
	}
	
	/**
	 * Creates and executes a MaltParser configuration
	 * 
	 * @throws MaltChainedException
	 */
	private void maltParser() throws MaltChainedException{
		try {
			Configuration config = null;
			if (optionManager.getOptionValue("config", "type") != null) {
				Class<?> clazz = (Class<?>)optionManager.getOptionValue("config", "type");
				config = (Configuration)clazz.newInstance();
				config.initialize(0);
				config.process();
				config.finalize();
			}	
		} catch (InstantiationException e) {
			throw new MaltChainedException("The system was not able to construct a configuration. ", e);
		} catch (IllegalAccessException e) {
			throw new MaltChainedException("The system was not able to construct a configuration. ", e);
		} 
	}
	
	/**
	 * Returns the option manager
	 * 
	 * @return the option manager
	 */
	public static OptionManager getOptionManager() {
		return optionManager;
	}
	/**
	 * Returns the plugin manager
	 * 
	 * @return the plugin manager
	 */
	public static PluginLoader getPluginLoader() {
		return pluginLoader;
	}
	/**
	 * Returns the system logger
	 * 
	 * @return the system logger
	 */
	public static Logger getSystemLogger() {
		return systemLogger;
	}

	/**
	 * Returns the system verbosity level
	 * 
	 * @return the system verbosity level
	 */
	public Level getSystemVerbosityLevel() {
		return systemVerbosityLevel;
	}

	/**
	 * Sets the system verbosity level
	 * 
	 * @param verbosity	a system verbosity level
	 */
	public void setSystemVerbosityLevel(String verbosity) {
		this.systemVerbosityLevel = Level.toLevel(verbosity, Level.INFO);
	}
	
	
	/**
	 * Returns the application header
	 * 
	 * @return the application header
	 */
	public static String header() {
		StringBuilder sb = new StringBuilder();
		sb.append(
				"-----------------------------------------------------------------------------\n"+
				"                          MaltParser "+version+"                             \n"+
				"-----------------------------------------------------------------------------\n"+
				"         MALT (Models and Algorithms for Language Technology) Group          \n"+
				"             School of Mathematics and Systems Engineering (MSI)             \n"+
				"                        Vaxjo University, Sweden                             \n"+
				"-----------------------------------------------------------------------------\n");
		return sb.toString();
	}
	
	/**
	 * Returns a short version of the help
	 * 
	 * @return a short version of the help
	 */
	private static String shortHelp() {
		StringBuilder sb = new StringBuilder();
		sb.append("\n"+
				"Usage: \n"+
				"   java -jar malt.jar -f <path to option file> <options>\n"+
				"   java -jar malt.jar -h for more help and options\n\n"+
				optionManager.getOptionDescriptions().toStringOptionGroup("system")+
				"Documentation: docs/index.html\n");
		return sb.toString();
	}
	
	/**
	 * Returns a set of attributes present in the jar manifest file
	 * 
	 * @return a set of attributes present in the jar manifest file
	 */
	public static Attributes getManifestAttributes() {
		return manifestAttributes;
	}
	
	/**
	 * Returns the version number as string
	 * 
	 * @return the version number as string
	 */
	public static String getVersion() {
		return version;
	}
	
	/**
	 * Returns the build date
	 * 
	 * @return the build date
	 */
	public static String getBuildDate() {
		return buildDate;
	}
	
	/**
	 * Loads the manifest attributes from the manifest in the jar-file
	 * 
	 * @throws MaltChainedException
	 */
	private void getManifestInfo() throws MaltChainedException {
	   try {
            URL codeBase = MaltConsoleEngine.class.getProtectionDomain().getCodeSource().getLocation();
            if(codeBase.getPath().endsWith(".jar")) {
            	JarFile jarfile = new JarFile(codeBase.getPath());
                Manifest manifest = jarfile.getManifest();
                Attributes manifestAttributes = manifest.getMainAttributes();
                version = manifestAttributes.getValue("Implementation-Version");
                buildDate = manifestAttributes.getValue("Build-Date");
            }
        } catch(IOException e) {
        	throw new MaltChainedException("Could not read the manifest of the malt jar file. ", e);
        }
	}
}
