package org.maltparser.core.dataformat;
/**
 *  DataFormatEntry is used for storing information about one column in a data format. 
 *
 * @author Johan Hall
 * @since 1.0
**/
public class DataFormatEntry implements Comparable<DataFormatEntry>{
	/**
	 * Column position from 0 to n-1, where n is number of columns in the data format.
	 */
	private Integer position;
	/**
	 * Column name
	 */
	private String name;
	/**
	 * Column category (INPUT, HEAD or OUTPUT)
	 */
	private String category;
	/**
	 * Column type (STRING, INTEGER, BOOLEAN, ECHO or IGNORE)
	 */
	private String type;
	/**
	 * Default output for a ignore column
	 */
	private String defaultOutput;
	
	/**
	 * Creates a data format entry
	 * 
	 * @param position	column position
	 * @param name	column name
	 * @param category	column category
	 * @param type	column type
	 * @param defaultOutput	default output for a ignore column
	 */
	public DataFormatEntry(int position, String name, String category, String type, String defaultOutput) {
		setPosition(position);
		setName(name);
		setCategory(category);
		setType(type);
		setDefaultOutput(defaultOutput);
	}
	
	/**
	 * Returns the column position
	 * 
	 * @return the column position
	 */
	public int getPosition() {
		return position;
	}

	/**
	 * Sets the column position
	 * 
	 * @param position	column position
	 */
	public void setPosition(int position) {
		this.position = position;
	}

	/**
	 * Returns the column name
	 * @return	the column name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the column name
	 * @param name	column name
	 */
	public void setName(String name) {
		this.name = name.toUpperCase();
	}

	/**
	 * Returns the column category
	 * 
	 * @return	the column category
	 */
	public String getCategory() {
		return category;
	}

	/**
	 * Sets the column category
	 * 
	 * @param category	the column category
	 */
	public void setCategory(String category) {
		this.category = category.toUpperCase();
	}

	/**
	 * Return the column type
	 * 
	 * @return	the column type
	 */
	public String getType() {
		return type;
	}

	/**
	 * Sets the column type
	 * 
	 * @param type	the column type
	 */
	public void setType(String type) {
		this.type = type.toUpperCase();
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(DataFormatEntry o) {
		return position.compareTo(o.getPosition());
	}

	/**
	 * Returns the default output of an ignore column
	 * 
	 * @return the default output of an ignore column
	 */
	public String getDefaultOutput() {
		return defaultOutput;
	}

	/**
	 * Sets the default output of an ignore column
	 * 
	 * @param defaultOutput  the default output of an ignore column
	 */
	public void setDefaultOutput(String defaultOutput) {
		this.defaultOutput = defaultOutput;
	}
	
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof DataFormatEntry)) {
			return false;
		}
		DataFormatEntry objC = (DataFormatEntry)obj;
		return ((name == null) ? objC.name == null : name.equals(objC.name)) &&
				((type == null) ? objC.type == null : type.equals(objC.type)) &&
				((category == null) ? objC.category == null : category.equals(objC.category)) && 
				((position == null) ? objC.position == null : position.equals(objC.position)) && 
				((defaultOutput == null) ? objC.defaultOutput == null : defaultOutput.equals(objC.defaultOutput));
	}

	public int hashCode() {
		int hash = 7;
		hash = 31 * hash + (null == name ? 0 : name.hashCode());
		hash = 31 * hash + (null == type ? 0 : type.hashCode());
		hash = 31 * hash + (null == category ? 0 : category.hashCode());
		hash = 31 * hash + (null == position ? 0 : position.hashCode());
		hash = 31 * hash + (null == defaultOutput ? 0 : defaultOutput.hashCode());
		return hash;
	}
	
	public String toString() {
		final StringBuilder sb = new StringBuilder();
		sb.append(position);
		sb.append("\t");
		sb.append(name);
		sb.append("\t");
		sb.append(category);
		sb.append("\t");
		sb.append(type);
		if (defaultOutput != null) {
			sb.append("\t");
			sb.append(defaultOutput);
		}
		return sb.toString();
	}
}
