package org.maltparser.core.dataformat;

import java.util.Iterator;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.symbol.SymbolTableHandler;

/**
 *  
 *
 * @author Johan Hall
 * @since 1.0
**/
public class DataFormatInstance implements Iterable<ColumnDescription> {
	private SortedSet<ColumnDescription> columnDescriptions;
	private SortedMap<String,ColumnDescription> headColumnDescriptions;
	private SortedMap<String,ColumnDescription> outputColumnDescriptions;
	private SortedMap<String,ColumnDescription> inputColumnDescriptions;
	private SortedMap<String,ColumnDescription> ignoreColumnDescriptions;
	private SymbolTableHandler symbolTables;
	
	public DataFormatInstance(SortedSet<DataFormatEntry> entries, SymbolTableHandler symbolTables, String specialSymbolsfileName, String rootLabel) throws MaltChainedException {
		columnDescriptions = new TreeSet<ColumnDescription>();
		headColumnDescriptions = new TreeMap<String,ColumnDescription>();
		outputColumnDescriptions = new TreeMap<String,ColumnDescription>();
		inputColumnDescriptions = new TreeMap<String,ColumnDescription>();
		ignoreColumnDescriptions = new TreeMap<String,ColumnDescription>();
		this.symbolTables = symbolTables;
		createColumnDescriptions(entries, specialSymbolsfileName, rootLabel);
	}
	
	
	private void createColumnDescriptions(SortedSet<DataFormatEntry> entries, String specialSymbolsfileName, String rootLabel) throws MaltChainedException {
		for (DataFormatEntry entry : entries) {
			ColumnDescription column = new ColumnDescription(entry.getPosition(),entry.getName(), entry.getCategory(), entry.getType(), entry.getDefaultOutput(), symbolTables, specialSymbolsfileName, rootLabel);
			columnDescriptions.add(column);
			if (column.getType() == ColumnDescription.IGNORE) {
				ignoreColumnDescriptions.put(column.getName(), column);
			} else if (column.getCategory() == ColumnDescription.HEAD) { 
				headColumnDescriptions.put(column.getName(), column);
			} else if (column.getCategory() == ColumnDescription.OUTPUT) {
				outputColumnDescriptions.put(column.getName(), column);
			} else if (column.getCategory() == ColumnDescription.INPUT) {
				inputColumnDescriptions.put(column.getName(), column);
			}
		}
	}
	
	public ColumnDescription getColumnDescriptionByName(String name) {
		for (ColumnDescription column : columnDescriptions) {
			if (column.getName().equals(name)) {
				return column;
			}
		}
		return null;
	}

	public int getNumberOfColumnDescriptions() {
		return columnDescriptions.size();
	}
	
	public Iterator<ColumnDescription> iterator() {
		return columnDescriptions.iterator();
	}
	
	public ColumnDescription getHeadColumnDescription() {
		return headColumnDescriptions.get(headColumnDescriptions.firstKey());
	}
	
	public SortedMap<String,ColumnDescription> getHeadColumnDescriptions() {
		return headColumnDescriptions;
	}

	public SortedMap<String,ColumnDescription> getOutputColumnDescriptions() {
		return outputColumnDescriptions;
	}

	public SortedMap<String,ColumnDescription> getInputColumnDescriptions() {
		return inputColumnDescriptions;
	}
	
	public SortedMap<String,ColumnDescription> getIgnoreColumnDescriptions() {
		return ignoreColumnDescriptions;
	}
	
	public SymbolTableHandler getSymbolTables() {
		return symbolTables;
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		for (ColumnDescription column : columnDescriptions) {
			sb.append(column);
			sb.append('\n');
		}
		return sb.toString();
	}
}
