package org.maltparser.parser.guide.feature;

import org.maltparser.core.dataformat.ColumnDescription;
import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.sentence.Token;
import org.maltparser.core.symbol.nullvalue.NullValues.NullValueId;
import org.maltparser.parser.algorithm.ParsingAlgorithm;


/**


@author Johan Hall
@since 1.0
 */
public final class InputColumnFeature extends ColumnFeature {
	public InputColumnFeature(ParsingAlgorithm parsingAlgorithm, String spec) throws MaltChainedException {
		super();
		super.initialize(parsingAlgorithm);
		spec = spec.trim();
		int index = spec.indexOf(',');
		if (index == -1) {
			throw new FeatureException("The column name cannot be found at feature line '"+spec+"'. ");
		}
		ColumnDescription column = parsingAlgorithm.getDependencyGraph().getDataFormatInstance().getColumnDescriptionByName(spec.substring(0, index));
		if (column == null) {
			throw new FeatureException("The column '"+spec.substring(0, index)+"' cannot be found. ");
		}
		if (column.getCategory() != ColumnDescription.INPUT) {
			throw new FeatureException("The column '"+column.getName()+"' is not an input column at line '"+spec+"'. ");
		}
		super.setColumn(column);
		if (!init(spec.substring(index+1))) { 
			throw new FeatureException("The input column feature could not be initialized at line '"+spec+"'. ");
		}
	}
	
	public boolean init(String spec) throws MaltChainedException {
		return super.init(spec);
	}
	
	public void update() throws MaltChainedException {
		featureValues.clear();
		featureKnown.clear();
		currentUpdateNode = getNode();
		if (currentUpdateNode != null) {
			Token token = currentUpdateNode.getToken();
			if (token != null) { // currentUpdateNode has no token because it is a root node
				featureValues.put(token.getColumnInstance(column).getIntegerValue(), token.getColumnInstance(column).getStringValue()); 
				featureKnown.put(token.getColumnInstance(column).getIntegerValue(), token.getColumnInstance(column).isKnown());
				nullValue = false;
			} else {
				featureKnown.put(column.getSymbolTable().getNullValueCode(NullValueId.ROOT_NODE), true);
				featureValues.put(column.getSymbolTable().getNullValueCode(NullValueId.ROOT_NODE), column.getSymbolTable().getNullValueSymbol(NullValueId.ROOT_NODE));
				nullValue = true;
			}
		} else {
			featureValues.put(column.getSymbolTable().getNullValueCode(NullValueId.NO_NODE), column.getSymbolTable().getNullValueSymbol(NullValueId.NO_NODE));
			featureKnown.put(column.getSymbolTable().getNullValueCode(NullValueId.NO_NODE), true);
			nullValue = true;
		}
	}
	
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!(obj instanceof InputColumnFeature)) {
			return false;
		}
		return obj.toString().equals(toString());
	}
	
	public int hashCode() {
		int hash = 7;
		hash = 31 * hash + (null == toString() ? 0 : toString().hashCode());
		return hash;
	}
	
	public String toString() {
		final StringBuilder sb = new StringBuilder();
		sb.append("InputColumn(");
		sb.append(super.toString());
		sb.append(')');
		return sb.toString();
	}
}
