package org.maltparser.parser.guide.predmodel;

import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import org.maltparser.core.config.SingleMaltConfiguration;
import org.maltparser.core.exception.MaltChainedException;
import org.maltparser.core.graph.DependencyGraph;
import org.maltparser.core.sentence.Sentence;
import org.maltparser.parser.algorithm.ParsingAlgorithm;
import org.maltparser.parser.algorithm.helper.ParserAction;
import org.maltparser.parser.guide.Guide;
import org.maltparser.parser.guide.classtable.AllCombinedClassTable;
import org.maltparser.parser.guide.classtable.ClassTable;
import org.maltparser.parser.guide.feature.Feature;
import org.maltparser.parser.guide.feature.FeatureVector;
import org.maltparser.parser.guide.instmodel.AtomicModel;
import org.maltparser.parser.guide.instmodel.FeatureDivideModel;
import org.maltparser.parser.guide.instmodel.InstanceModel;
/**
The combined prediction guide model combines the transition and all output columns into one
class sets (the same behavior as MaltParser 0.4). 

@author Johan Hall
@since 1.0
*/
public class CombinedPredictionModel implements PredictionModel {
	protected Guide guide; 
	protected String modelName;
	protected FeatureVector features;
	protected ClassTable classCodeTable;
	protected InstanceModel model;
	
	/**
	 * Constructs a combined prediction guide model.
	 * 
	 * @param guide a Guide object
	 * @throws MaltChainedException
	 */
	public CombinedPredictionModel(Guide guide) throws MaltChainedException {
		setGuide(guide);
		setModelName(getConfiguration().getName());
		initFeatures();
		initClassCodeTable();
		initModel();
	}
	
	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.model.Model#addInstance(org.maltparser.parser.algorithm.helper.ParserAction)
	 */
	public void addInstance(ParserAction parserAction) throws MaltChainedException {
		if (model == null || getParserMode() != SingleMaltConfiguration.LEARN) {
			throw new ModelException("The combined prediction model cannot add instance. ");
		}
		update();
		classCodeTable.updateClassCode(); 
		model.addInstance(parserAction);
	}
	
	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.model.Model#finishedSentence(org.maltparser.core.sentence.Sentence, org.maltparser.core.graph.DependencyGraph)
	 */
	public void finalizeSentence(Sentence sentence, DependencyGraph dependencyGraph) throws MaltChainedException {
		if (model == null || getParserMode() != SingleMaltConfiguration.LEARN) {
			throw new ModelException("The combined prediction model cannot finish sentence. ");
		}
		model.finalizeSentence(sentence, dependencyGraph);
	}
	
	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.model.Model#finishedAllData()
	 */
	public void noMoreInstances() throws MaltChainedException {
		if (model == null || getParserMode() != SingleMaltConfiguration.LEARN) {
			throw new ModelException("The combined prediction model cannot finish up all data. ");
		}
		model.noMoreInstances();
		model.train();
	}
	
	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.model.Model#terminate()
	 */
	public void finalize() throws MaltChainedException {
		if (model == null || getParserMode() != SingleMaltConfiguration.LEARN) {
			throw new ModelException("The combined prediction model cannot terminate. ");
		}
		model.finalize();
	}
	
	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.model.Model#predict()
	 */
	public boolean predict() throws MaltChainedException {
		if (model == null || getParserMode() != SingleMaltConfiguration.PARSE) {
			throw new ModelException("The combined prediction model cannot predict. ");
		}
		update();
		model.predict();
		if (classCodeTable.updateParserActionWithFirstKBest() == null) {
			return false;
		}
		return true;
	}
	
	public boolean predictFromKBestList() throws MaltChainedException {
		if (getParserMode() != SingleMaltConfiguration.PARSE) {
			throw new ModelException("The combined prediction model cannot predict. ");
		}
		
		if (classCodeTable.updateParserActionWithNextKBest() == null) {
			return false;
		}
		return true;
	}
	
	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.model.Model#update()
	 */
	public void update() throws MaltChainedException {
		for (Feature feature : features) {
			feature.update();
		}
	}
	
	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.model.Model#updateCardinality()
	 */
	public void updateCardinality() throws MaltChainedException {
		for (Feature feature : features) {
			feature.updateCardinality();
		}
	}
	
	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.model.Model#getModelName()
	 */
	public String getModelName() {
		return modelName;
	}

	/**
	 * Sets the model name 
	 * 
	 * @param modelName the model name
	 */
	protected void setModelName(String modelName) {
		this.modelName = modelName;
	}

	/**
	 * Returns the feature vector
	 * 
	 * @return the feature vector
	 */
	public FeatureVector getFeatures() {
		return features;
	}

	/**
	 * Initialize the feature vector and reads the feature specification file according to 
	 * the <code>--guide-features</code> option or reads the default feature model 
	 * specification for current parsing algorithm.
	 * 
	 * @throws MaltChainedException
	 */
	protected void initFeatures() throws MaltChainedException {
		features = new FeatureVector(guide.getParsingAlgorithm());
		
		features.readFeatureSpecification(getConfiguration().getOptionValue("guide", "features").toString());
		if (getConfiguration().getMode() == SingleMaltConfiguration.LEARN) {
			try {
				getConfiguration().getInfoFileWriter().write("\nFEATURE MODEL\n");
				getConfiguration().getInfoFileWriter().write(features.toString());
				getConfiguration().getInfoFileWriter().flush();
			} catch (IOException e) {
				throw new ModelException("Could not write feature model specification to configuration information file. ", e);
			}
		}
		
	}

	/**
	 * Returns the all combined class table
	 * @see org.maltparser.parser.guide.classtable.AllCombinedClassTable
	 * @return the all combined class table
	 */
	public ClassTable getClassCodeTable() {
		return classCodeTable;
	}

	/**
	 * Initialize the class code table. The combined guide model combines the transition and output labels
	 * into one class table by using the AllCombinedClassTable.
	 * 
	 * @see org.maltparser.parser.guide.classtable.AllCombinedClassTable
	 * @throws MaltChainedException
	 */
	protected void initClassCodeTable() throws MaltChainedException {
		this.classCodeTable = new AllCombinedClassTable(getConfiguration().getSymbolTables(), 
												"AllCombinedClassTable", 
												guide.getParsingAlgorithm().getParserAction(), 
												getConfiguration().getOptionValue("guide", "classitem_separator").toString().charAt(0),
												(Integer)getConfiguration().getOptionValue("guide", "kbest"));
	}

	/**
	 * Initialize instance guide model according to the <code>--guide-data_split_column</code>. If the option
	 * is empty, when initialize a single model. Otherwise, a data split model (which will split the training instance 
	 * according to a feature)
	 * 
	 * @throws MaltChainedException
	 */
	protected void initModel() throws MaltChainedException {
		if (getConfiguration().getOptionValue("guide", "data_split_column").toString().length() == 0) {
			model = new AtomicModel(-1, features, classCodeTable, this);
		} else {
			model = new FeatureDivideModel(features, classCodeTable, this);
		}
	}
	
	public OutputStreamWriter getOutputStreamWriter(String suffix) throws MaltChainedException {
		if (guide != null) {
			return guide.getOutputStreamWriter(suffix);
		}
		return null;
	}
	
	public InputStreamReader getInputStreamReader(String suffix) throws MaltChainedException {
		if (guide != null) {
			return guide.getInputStreamReader(suffix);
		}
		return null;
	}
	
	public File getFile(String suffix) throws MaltChainedException {
		if (guide != null) {
			return guide.getFile(suffix);
		}
		return null;
	}
	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.model.Model#getConfiguration()
	 */
	public SingleMaltConfiguration getConfiguration() throws MaltChainedException {
		return guide.getConfiguration();
	}
	

	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.GuideModel#getParsingAlgorithm()
	 */
	public ParsingAlgorithm getParsingAlgorithm() {
		return guide.getParsingAlgorithm();
	}
	
	/**
	 * Sets a Guide object
	 * 
	 * @param guide a Guide object
	 */
	protected void setGuide(Guide guide) {
		this.guide = guide;
	}

	/* (non-Javadoc)
	 * @see org.maltparser.parser.guide.model.Model#getParserMode()
	 */
	public int getParserMode() throws MaltChainedException {
		if (getConfiguration() != null) {
			return getConfiguration().getMode();
		}
		return -1;
	}
	
	public String toString() {
		final StringBuilder sb = new StringBuilder();
		//TODO
		return sb.toString();
	}
}
